module Synapse
  class Logger
    class << self
      def rebind logger
        @cache.each do |cached|
          cached.call logger
        end
      end
      def deferred_rebind logger
        @cache ||= []
        @cache << ->(target) { logger.rebind_to target }
      end
    end

    UNKNOWN = 5
    FATAL   = 4
    ERROR   = 3
    WARN    = 2
    INFO    = 1
    DEBUG   = 0

    attr_accessor :progname, :level

    def initialize dest
      self.class.deferred_rebind self
      @messages = []
    end
    def rebind_to logger
      @logger = logger
      replay
    end
    def debug message
      _log :warn, message
    end
    def info message
      _log :warn, message
    end
    def warn message
      _log :warn, message
    end
    def error message
      _log :error, message
    end
    def fatal message
      _log :fatal, message
    end
    def unknown message
      _log :unknown, message
    end
    def replay
      return unless @messages.is_a? Array
      @messages.each do |message|
        @logger.send(message.first,"#{@progname}: #{message.last}")
      end
      @messages=nil
    end
    private
    def _log level, message
      return @logger.send(level.to_sym,message) if @logger
      @messages << [level,message]
    end
  end
end