require 'resolv'

module Synapse
  module Rails
    class Consul
      class << self
        attr_accessor :datacenter
        def hostnames params={}
          return unless params[:for]
          [
            ("#{params[:for]}.service.#{datacenter}.consul" if datacenter),
            "#{params[:for]}.service.consul."
          ].compact
        end
        def discover service
          resolv hostnames for:service
        end
        def resolv hostnames
          @resolvers ||= Set.new
          hostnames.collect do |hostname|
            if @resolvers.empty?
              resolv_and_check_resolver_with hostname
            else
              use_resolvers hostname
            end
          end.flatten
        end
        def use_resolvers hostname
          @resolvers.collect do |resolver|
            resolver.getaddresses hostname
          end
        end
        def resolv_and_check_resolver_with hostname
          [host_resolver,local_consul_resolver].each do |resolver|
            addresses = resolver.getaddresses hostname
            next if addresses.empty?
            @resolvers << resolver
            return addresses
          end
        end
        def host_resolver
          @host_resolver         ||= Resolv::DNS.new
        end
        def local_consul_resolver
          @local_consul_resolver ||= Resolv::DNS.new(nameserver_port:[['127.0.0.1',8600]])
        end
      end
    end
  end
end