require 'fileutils'

module Synapse
  module Rails
    module Haproxy
      attr_writer :reload_command, :config_file_path, :socket_file_path, :pid_file_path, :do_writes, :do_reloads, :do_socket, :global, :defaults, :extra_sections
      def reload_command
        @reload_command || "haproxy -f '#{config_file_path}' -p '#{pid_file_path}' -st $(cat '#{pid_file_path}' 2>/dev/null|| true)"
      end
      def instance_name
        "synapse_#{instance}"
      end
      def workdir
        @workdir ||= ::Rails.root unless ::Rails.root.to_s.size > 42
        @workdir ||= Pathname.new "/tmp"       
      end
      def config_file_path
        @config_file_path ||= workdir.join("tmp","#{instance_name}.cfg").to_s
        FileUtils.mkdir_p File.dirname @config_file_path
        @config_file_path
      end
      def socket_file_path 
        @socket_file_path ||= workdir.join("tmp","sockets","#{instance_name}.sock").to_s
        FileUtils.mkdir_p File.dirname @socket_file_path
        @socket_file_path
      end
      def pid_file_path
        @pid_file_path ||= workdir.join("tmp","pids","#{instance_name}.pid").to_s
        FileUtils.mkdir_p File.dirname @pid_file_path
        @pid_file_path
      end
      def do_writes
        @do_writes || true
      end
      def do_reloads
        @do_reloads || true
      end
      def do_socket
        @do_socket || true
      end
      def global
        @global || ["daemon","maxconn 4096","stats socket #{socket_file_path} mode 666 level admin"]
      end
      def defaults
        @defaults || ["log global","option dontlognull","maxconn 1024","retries 3","timeout connect 5s","timeout client 60s","timeout server 60s","option redispatch","balance roundrobin"]
      end
      def extra_sections
        @extra_sections || {
          "listen stats 127.0.0.1:#{Freeport.port}" => [
            "mode http",
            "stats enable",
            "stats uri /",
            "stats refresh 5s"
          ]
        }
      end
      def instance
        @instance ||= SecureRandom.hex(3)
      end
      def haproxy
        %i{reload_command config_file_path socket_file_path do_writes do_reloads do_socket global defaults extra_sections}.collect do |section|
          [section,self.send(section)]
        end.to_h.with_indifferent_access
      end
    end
  end
end