module Synapse
  module Rails
    module Linker
      attr_accessor :synapse_instance
      def add_service_provider provider
        @services ||= []
        @services << provider
        self.define_singleton_method(provider.service_name.to_sym){provider}
        unless self.respond_to? (provider.service.to_sym)
          self.define_singleton_method(provider.service.to_sym){provider}
        end
      end
      def services
        @services.collect do |service|
          [service.service_name,service.synapse]
        end.to_h.with_indifferent_access
      end
      def synapse
        @synapse ||= {
          services:services,
          haproxy:haproxy
        }.with_indifferent_access unless @started
        @synapse
      end
      def prepare
        @synapse_instance ||= Synapse.new synapse
      end
      def wait
        loop do
          break unless @synapse_instance.instance_variable_get("@haproxy").instance_variable_get("@restart_required")
          sleep 0.01
        end
      end
      def started?
        @started || false
      end
      def start
        prepare
        @started = true
        @marshal = Thread.new do
          synapse_instance.run
        end
        at_exit do
          log = ::Rails.logger if defined? ::Rails.logger
          if File.exists? pid_file_path
            pids = File.read(pid_file_path).strip
            cmd  = "kill -9 #{pids}"
            log.info "HAProxy stop #{pids}" if log
            `#{cmd}`
            log.debug "HAProxy unlink #{pid_file_path}" if log
            File.unlink pid_file_path
          end
          if File.exists? socket_file_path
            log.debug  "HAProxy unlink #{socket_file_path}" if log
            File.unlink socket_file_path
          end
          if File.exists? config_file_path
            log.debug  "HAProxy unlink #{config_file_path}" if log
            File.unlink config_file_path
          end
        end
        wait
      end
    end
  end
end