module Synapse
  module Rails
    module Providers
      class Base
        class << self
          attr_accessor :service, :tester
          attr_writer :base_path
          def base_path
            @base_path ||= "/nerve"
          end
          def hosts
            @hosts ||= ::Synapse::Rails::Consul.discover(:zookeeper).collect do |address|
              "#{address}:#{ZK.default_port}"
            end
          end
        end
        attr_accessor :function, :check
        def initialize params={}
          @function        = "common"
          @default_servers = []
          config params
        end
        def path
          [base_path,::Synapse::Tenancy.customer,service,function,"services"].join("/")
        end
        def hosts
          @hosts ||= self.class.hosts
        end
        def discovery
          %i{method path hosts}.collect do |element|
            [element,self.send(element)]
          end.to_h.with_indifferent_access
        end
        def haproxy
          unless ::Synapse::Rails.started?
            @haproxy = ::Synapse::Rails::ProviderProxy.new(check:@check).haproxy
          end
          @haproxy
        end
        def default_servers
          @default_servers
        end
        def synapse
          {
            default_servers:default_servers,
            discovery: discovery,
            haproxy: haproxy,
          }.with_indifferent_access
        end
        def config params={}
          params.each do |k,v|
            self.send(:"#{k}=",v) rescue nil
          end
        end
        def service_name
          [service,function].join("_")
        end
        def service
          self.class.service
        end
        def host
          "127.0.0.1"
        end
        def port
          @haproxy[:port]
        end
        protected
        def method
          "zookeeper"
        end
        def base_path
          self.class.base_path
        end
      end
    end
  end
end